@@ -1,7 +1,5 @@
 /* size.c -- report size of various sections of an executable file.
-   Copyright 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000,
-   2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
-   Free Software Foundation, Inc.
+   Copyright (C) 1991-2014 Free Software Foundation, Inc.
 
    This file is part of GNU Binutils.
 
@@ -191,9 +189,7 @@ avr_device_t avr[] =
 	{"at90usb646",    AVR64K,  AVR4K,  AVR2K},
 	{"at90usb647",    AVR64K,  AVR4K,  AVR2K},
 	{"ata5505",       AVR16K,  AVR512, AVR512},
-	{"ata5702m322",   AVR64K,  AVR1K,  AVR2K},
 	{"ata5790",       AVR16K,  AVR512, AVR2112},
-	{"ata5790n",      AVR16K,  AVR512, AVR2112},
 	{"ata5795",       AVR8K,   AVR512, AVR2112},
 	{"ata5272",       AVR8K,   AVR512, AVR512},
 	{"atmega64",      AVR64K,  AVR4K,  AVR2K},
@@ -217,7 +213,6 @@ avr_device_t avr[] =
 	{"atmega6490p",   AVR64K,  AVR4K,  AVR2K},
 	{"atmega64c1",    AVR64K,  AVR4K,  AVR2K},
 	{"atmega64hve",   AVR64K,  AVR4K,  AVR1K},
-	{"atmega64hve2",  AVR64K,  AVR4K,  AVR1K},
 	{"atmega64m1",    AVR64K,  AVR4K,  AVR2K},
 	{"m3000",         AVR64K,  AVR4K,  0UL},
 
@@ -225,9 +220,7 @@ avr_device_t avr[] =
 
 	{"atxmega32a4",   AVR36K,  AVR4K,  AVR1K},
 	{"atxmega32a4u",  AVR36K,  AVR4K,  AVR1K},
-	{"atxmega32c3",   AVR36K,  AVR4K,  AVR1K},
 	{"atxmega32c4",   AVR36K,  AVR4K,  AVR1K},
-	{"atxmega32d3",   AVR36K,  AVR4K,  AVR1K},
 	{"atxmega32d4",   AVR36K,  AVR4K,  AVR1K},
 	{"atxmega32e5",   AVR36K,  AVR4K,  AVR1K},
 	{"atxmega16e5",   AVR20K,  AVR2K,  AVR512},
@@ -235,7 +228,6 @@ avr_device_t avr[] =
 
 	{"at90can32",     AVR32K,  AVR2K,  AVR1K},
 	{"at94k",         AVR32K,  AVR4K,  0UL},
-	{"ata6614q",      AVR32K,  AVR2K,  AVR1K},
 	{"atmega32",      AVR32K,  AVR2K,  AVR1K},
 	{"atmega32a",     AVR32K,  AVR2K,  AVR1K},
 	{"atmega323",     AVR32K,  AVR2K,  AVR1K},
@@ -275,17 +267,11 @@ avr_device_t avr[] =
 	{"atxmega16c4",   AVR20K,  AVR2K,  AVR1K},
 	{"atxmega16d4",   AVR20K,  AVR2K,  AVR1K},
 
-	{"ata5831",       AVR20K,  AVR1K,  AVR1K},
-	{"ata5782",       AVR20K,  AVR1K,  AVR1K},
-
 	{"at76c711",      AVR16K,  AVR2K,  0UL},
 	{"at90pwm161",    AVR16K,  AVR1K,  AVR512},
 	{"at90pwm216",    AVR16K,  AVR1K,  AVR512},
 	{"at90pwm316",    AVR16K,  AVR1K,  AVR512},
 	{"at90usb162",    AVR16K,  AVR512, AVR512},
-	{"ata6613c",      AVR16K,  AVR1K,  AVR512},
-	{"ata6617c",      AVR16K,  AVR512, AVR512},
-	{"ata664251",     AVR16K,  AVR512, AVR512},
 	{"atmega16",      AVR16K,  AVR1K,  AVR512},
 	{"atmega16a",     AVR16K,  AVR1K,  AVR512},
 	{"atmega161",     AVR16K,  AVR1K,  AVR512},
@@ -330,8 +316,6 @@ avr_device_t avr[] =
 	{"ata6285",       AVR8K,   AVR512, 320UL},
 	{"ata6286",       AVR8K,   AVR512, 320UL},
 	{"ata6289",       AVR8K,   AVR512, 320UL},
-	{"ata6612c",      AVR8K,   AVR1K,  AVR512},
-	{"ata6616c",      AVR8K,   AVR512, AVR512},
 	{"atmega8",       AVR8K,   AVR1K,  AVR512},
 	{"atmega8a",      AVR8K,   AVR1K,  AVR512},
 	{"atmega8515",    AVR8K,   AVR512, AVR512},
@@ -344,7 +328,6 @@ avr_device_t avr[] =
 	{"atmega8u2",     AVR8K,   AVR512, AVR512},
 	{"attiny84",      AVR8K,   AVR512, AVR512},
 	{"attiny84a",     AVR8K,   AVR512, AVR512},
-	{"attiny841",	  AVR8K,   AVR512, AVR512},
 	{"attiny85",      AVR8K,   AVR512, AVR512},
 	{"attiny861",     AVR8K,   AVR512, AVR512},
 	{"attiny861a",    AVR8K,   AVR512, AVR512},
@@ -362,7 +345,6 @@ avr_device_t avr[] =
 	{"attiny43u",     AVR4K,   AVR256, AVR64},
 	{"attiny44",      AVR4K,   AVR256, AVR256},
 	{"attiny44a",     AVR4K,   AVR256, AVR256},
-	{"attiny441",	  AVR4K,   AVR256, AVR256},	
 	{"attiny45",      AVR4K,   AVR256, AVR256},
 	{"attiny461",     AVR4K,   AVR256, AVR256},
 	{"attiny461a",    AVR4K,   AVR256, AVR256},
